using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace VCD_Simple
{
    public partial class Form1 : Form
    {
        // These variables will hold the interfaces to the Gain property
        private TIS.Imaging.VCDRangeProperty GainRange;
        private TIS.Imaging.VCDSwitchProperty GainSwitch;
        private TIS.Imaging.VCDAbsoluteValueProperty GainValue;

        private TIS.Imaging.VCDSwitchProperty ExposureSwitch;
        private TIS.Imaging.VCDAbsoluteValueProperty ExposureValue;

        private TIS.Imaging.VCDSwitchProperty WhiteBalanceAutoSwitch;
        private TIS.Imaging.VCDMapStringsProperty WhiteBalancMapStrings;
        private TIS.Imaging.VCDButtonProperty SoftwareTrigger;

        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Form1_Load
        ///
        /// Check whether a device has been specified in the properties of IC Imaging
        /// Control. If there is no device, show the device selection dialog.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Form1_Load(object sender, EventArgs e)
        {

            sldGain.Enabled = false;
            chkGainAuto.Enabled = false;
            cmb_whiteBalance.Enabled = false;


            if (icImagingControl1.DeviceValid == false)
            {
                icImagingControl1.ShowDeviceSettingsDialog();
            }

            if (icImagingControl1.DeviceValid)
            {

                if (GetWhiteBalanceItem() == true)
                {
                    cmb_whiteBalance.Enabled = true;
                    chk_WhiteBalanceAuto.Checked = WhiteBalanceAutoSwitch.Switch;
                    foreach (string Item in WhiteBalancMapStrings.Strings)
                    {
                        cmb_whiteBalance.Items.Add(Item);
                    }

                }

                chkExposureAuto.Checked = false;
                icImagingControl1.DeviceTrigger = false; //Triggermode OFF
                icImagingControl1.LiveCaptureContinuous = false;
                icImagingControl1.LiveCaptureLastImage = true;
                icImagingControl1.LiveDisplay = true;
                button1.Enabled = false;

                if (GetGainItem() == true & GetExposureItem() == true)
                {
                    // Initialize the slider with the current range and value of the
                    // GainRange object.
                    sldGain.Enabled = true;
                    sldGain.Minimum = GainRange.RangeMin;
                    sldGain.Maximum = GainRange.RangeMax;
                    sldGain.Value = GainRange.Value;

                    // Initialize the checkbox with the GainSwitch object
                    if (GainSwitch != null & ExposureSwitch != null)
                    {
                        chkExposureAuto.Enabled = true;   
                        chkExposureAuto.Checked = ExposureSwitch.Switch;

                        chkGainAuto.Enabled = true;
                        sldGain.Enabled = !GainSwitch.Switch;
                        chkGainAuto.Checked = GainSwitch.Switch;
                    }
                    icImagingControl1.LiveStart();
                }
            }
        }

        /// <summary>
        /// GetGainItem
        ///
        /// Retrieve the Gain VCDPropertyItem and assign GainSwitch and GainRange.
        /// The function returns true, if the property exists. If the property does not
        /// exists, the function returns false.
        /// </summary>
        /// <returns></returns>
        private bool GetGainItem()
        {
            bool bPropertyFound = false;
            // Try Find Gain property in the VCDPropertyItems collection.
            // If Gain is not support by the current video capture device,
            // an excpetion is thrown.
            TIS.Imaging.VCDPropertyItem Gain = null;
            Gain = icImagingControl1.VCDPropertyItems.FindItem(TIS.Imaging.VCDIDs.VCDID_Gain);
            if (Gain != null)
            {
                bPropertyFound = true;
                // Acquire interfaces to the range and switch interface for value and auto
                GainRange = (TIS.Imaging.VCDRangeProperty)Gain.Elements.FindInterface(
                                                                    TIS.Imaging.VCDIDs.VCDElement_Value + ":" +
                                                                    TIS.Imaging.VCDIDs.VCDInterface_Range);
                GainSwitch = (TIS.Imaging.VCDSwitchProperty)Gain.Elements.FindInterface(
                                                                    TIS.Imaging.VCDIDs.VCDElement_Auto + ":" +
                                                                    TIS.Imaging.VCDIDs.VCDInterface_Switch);
                GainValue = (TIS.Imaging.VCDAbsoluteValueProperty)Gain.Elements.FindInterface(
                                                                    TIS.Imaging.VCDIDs.VCDElement_Value + ":" +
                                                                    TIS.Imaging.VCDIDs.VCDInterface_AbsoluteValue);

                textGain.Text = (GainValue.Value * 100).ToString();

                if (GainSwitch == null)
                {
                    MessageBox.Show("Automation of Gain is not supported by the current device!");
                }
            }

            // Show a message box, if Gain is not supported.
            if (Gain == null)
            {
                MessageBox.Show("Gain property is not supported by the current device!");
            }
            return bPropertyFound;
        }

        private bool GetExposureItem()
        {
            bool bPropertyFound = false;
            // Try Find Exposure property in the VCDPropertyItems collection.
            // If Gain is not support by the current video capture device,
            // an excpetion is thrown.
            TIS.Imaging.VCDPropertyItem Exposure = null;
            Exposure = icImagingControl1.VCDPropertyItems.FindItem(TIS.Imaging.VCDIDs.VCDID_Exposure);
            if (Exposure != null)
            {
                bPropertyFound = true;
                ExposureSwitch = (TIS.Imaging.VCDSwitchProperty)Exposure.Elements.FindInterface(
                                                                    TIS.Imaging.VCDIDs.VCDElement_Auto + ":" +
                                                                    TIS.Imaging.VCDIDs.VCDInterface_Switch);

                ExposureValue = (TIS.Imaging.VCDAbsoluteValueProperty)Exposure.Elements.FindInterface(
                                                                    TIS.Imaging.VCDIDs.VCDElement_Value + ":" +
                                                                    TIS.Imaging.VCDIDs.VCDInterface_AbsoluteValue);

                
                if (ExposureSwitch == null)
                {
                    MessageBox.Show("Automation of Exposure is not supported by the current device!");
                }
            }

            // Show a message box, if Gain is not supported.
            if (Exposure == null)
            {
                MessageBox.Show("Exposure property is not supported by the current device!");
            }
            return bPropertyFound;
        }


        private bool GetSoftwareTriggerItem()
        {
            bool bPropertyFound = false;



            SoftwareTrigger = (TIS.Imaging.VCDButtonProperty)icImagingControl1.VCDPropertyItems.
                                                    FindInterface(TIS.Imaging.VCDIDs.VCDID_TriggerMode,
                                                    TIS.Imaging.VCDIDs.VCDElement_SoftwareTrigger,
                                                    TIS.Imaging.VCDIDs.VCDInterface_Button);

            // Show a message box, if Gain is not supported.
            if (SoftwareTrigger == null)
            {
                bPropertyFound = false;
                MessageBox.Show("Exposure property is not supported by the current device!");
            }
            else
            {
                bPropertyFound = true;
            }
            return bPropertyFound;
        }
        private bool GetWhiteBalanceItem()
        {
            bool bPropertyFound = false;

            TIS.Imaging.VCDPropertyItem WhiteBalance = null;

            WhiteBalance = icImagingControl1.VCDPropertyItems.FindItem(TIS.Imaging.VCDIDs.VCDID_WhiteBalance);
            if (WhiteBalance != null)
            {
                bPropertyFound = true;
                WhiteBalancMapStrings = (TIS.Imaging.VCDMapStringsProperty)WhiteBalance.Elements.FindInterface(
                                                                    TIS.Imaging.VCDIDs.VCDElement_WhiteBalanceMode + ":" +
                                                                    TIS.Imaging.VCDIDs.VCDInterface_MapStrings);

                WhiteBalanceAutoSwitch = (TIS.Imaging.VCDSwitchProperty)WhiteBalance.Elements.FindInterface(
                                                                    TIS.Imaging.VCDIDs.VCDElement_Auto + ":" +
                                                                    TIS.Imaging.VCDIDs.VCDInterface_Switch);
                if (WhiteBalancMapStrings == null)
                {
                    MessageBox.Show("Palameter of WhiteBalance is not supported by the current device!");
                }
            }

            // Show a message box, if Gain is not supported.
            if (WhiteBalancMapStrings == null)
            {
                MessageBox.Show("WhiteBalance property is not supported by the current device!");
            }
            return bPropertyFound;
        }
        /// <summary>
        /// chkGainAuto_CheckedChanged
        ///
        /// Enable or disable the autmatic of the Gain property when the checkbox
        /// has been clicked.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkGainAuto_CheckedChanged(object sender, EventArgs e)
        {
            GainSwitch.Switch = chkGainAuto.Checked;
            sldGain.Enabled = !chkGainAuto.Checked;

            textGain.Text = (GainValue.Value * 100).ToString();
            
        }

        /// <summary>
        /// sldGain_Scroll
        ///
        /// Set the Gain to the current slider position.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void sldGain_Scroll(object sender, EventArgs e)
        {
            GainRange.Value = sldGain.Value;
            GainValue.Value = (GainValue.Value);
            textGain.Text = (GainValue.Value * 100).ToString();
        }

        private void TextGain_TextChanged(object sender, EventArgs e)
        {
            try
            {
               
            }
            catch (TIS.Imaging.ICException)
            {

            }
        }
        private void cmb_whiteBalance_SelectedIndexChanged(object sender, EventArgs e)
        {
            Object selectedItem = cmb_whiteBalance.SelectedItem;
            WhiteBalancMapStrings.String = selectedItem.ToString();
        }

        private void chk_WhiteBalanceAuto_CheckedChanged(object sender, EventArgs e)
        {
            WhiteBalanceAutoSwitch.Switch = chk_WhiteBalanceAuto.Checked;
        }

        private void Button1_Click(object sender, EventArgs e)
        {
            SoftwareTrigger.Push(); //@\tgEFAgK[  
        }

        private void IcImagingControl1_ImageAvailable(object sender, TIS.Imaging.ICImagingControl.ImageAvailableEventArgs e)
        {
            TIS.Imaging.ImageBuffer CurrentBuffer = null;
            CurrentBuffer = icImagingControl1.ImageBuffers[e.bufferIndex];
            icImagingControl1.DisplayImageBuffer(CurrentBuffer);

        }



        private void ChkExposureAuto_CheckedChanged(object sender, EventArgs e)
        {
            ExposureSwitch.Switch = chkExposureAuto.Checked;

        }



        private void chkTriggerMode_CheckedChanged(object sender, EventArgs e)
        {

            icImagingControl1.LiveStop();

            if (chkTriggerMode.Checked && GetSoftwareTriggerItem() == true)
            {
                icImagingControl1.DeviceTrigger = true; //Triggermode ON
                icImagingControl1.LiveCaptureContinuous = true;
                icImagingControl1.LiveCaptureLastImage = false;
                icImagingControl1.LiveDisplay = false;
                button1.Enabled = true;
            }
            else
            {
                icImagingControl1.DeviceTrigger = false; //Triggermode OFF
                icImagingControl1.LiveCaptureContinuous = false;
                icImagingControl1.LiveCaptureLastImage = true;
                icImagingControl1.LiveDisplay = true;
                button1.Enabled = false;
            }
            icImagingControl1.LiveStart();
        }
    }
}